@@ -23,10 +23,14 @@ class SinglyLinkedList:
 
     __slots__ = ('_head', '_tail', '_target',)
 
-    def __init__(self) -> None:
+    def __init__(self, values=None) -> None:
         self._head: Optional[SinglyNode] = None
         self._tail: Optional[SinglyNode] = None
         self._target: Optional[Any|SinglyNode] = None
+        if values:
+            for value in values:
+                self.tail = value
+
 
     @property
     def head(self) -> SinglyNode | None:
@@ -128,34 +132,24 @@ class SinglyLinkedList:
         return self._target
 
     @target.setter
-    def target(self, target_data: Any | SinglyNode) -> None:
+    def target(self, target_data: Any | SinglyNode):
         """Setter property for the target node data of the linked list.
 
         Args:
             target (Any): The data for which to match to node data.
         """
 
-        match target_data:
-            case _:
-                #  TODO: data validation
-                validated_data = target_data
-
         match self._head:
             case None:
-                #print(f"Empty `SinglyLinkedList()`. Target data is assigned to {type(self).__name__}'s target property.")
-                self._target = validated_data
+                raise ValueError(f"Cannot assign target data to empty {type(self).__name__}.")
 
             case self._head:
-                current_node = self._head
-                while current_node:
-                    if current_node.data == validated_data:
-                        #print(f"""At least one target match. First target node instance is assigned to {type(self).__name__}'s target property.""")
-                        setattr(self, '_target', current_node)
-                        return
-                    current_node = current_node.next
-                #print(f"""No target matches. Target data assigned to {type(self).__name__}'s target property.""")
-                setattr(self, '_target', validated_data)
-                return
+                if target_data:
+                    #  TODO: data validation
+                    validated_data = target_data
+                    self._target = validated_data
+                    return
+                raise ValueError("Target data unacceptable.")
 
 
     @target.deleter
@@ -238,16 +232,25 @@ class SinglyLinkedList:
         )
 
     def __repr__(self) -> str:
-        return f"{type(self).__name__}(head={self.head}, tail={self.tail}, target={self.target})"
+        #return f"{type(self).__name__}(head={type(self.head).__name__}, tail={self.tail})"
+        if not self._head:
+            return "SinglyLinkedList()"
+        node = self._head
+        nodes = []
+        while node:
+            nodes.append(repr(node.data))
+            node = node.next
+        return f"{type(self).__name__}([" + ', '.join(nodes) + "])"
 
     def __str__(self) -> str:
-        match self:
-            case SinglyLinkedList(head=None, tail=None):
-                return "This instance of SinglyLinkedList is empty."
-            case SinglyLinkedList(head=head, tail=tail):
-                if head is tail:
-                    return "This instance of SinglyLinkedList has a single node."
-                return "The head and tail are different nodes."
+        if not self._head:
+            return "Empty Singly Linked List"
+        node = self._head
+        nodes = []
+        while node:
+            nodes.append(str(node.data))
+            node = node.next
+        return ' -> '.join(nodes)
             
     def __dir__(self) -> list[str]:
         # Get the list of attributes and methods from the parent classes
@@ -351,7 +354,56 @@ class SinglyLinkedList:
         """
         self.tail = data  # trigger the tail setter
 
-    def search(self, target_data) -> bool:
+    def insert_after_target(self, target_data: Any, data_to_insert: Any) -> None:
+        """
+        steps:
+        1) the target data is validated
+        2) the head data is checked for the target
+            - if True, the linked list is checked if it's a one node list
+                - if True, simply use the tail property to insert after the head
+                - if False, the list contains more than one node
+        3) the tail data is checked for the target
+            - if True, simply use
+        """
+        self.target = target_data  # trigger the setter
+        # insert after the first encounter
+        if getattr(self._head, 'data') == self._target:
+            if self._head is self._tail:  # it's a one node list
+                self.tail = data_to_insert  # trigger the setter, tail property will validate input
+                return
+        current_node = getattr(self, '_head')  # traversal
+        while current_node is not self._tail:  # traversal
+            if current_node.data == self._target:
+                new_node = SinglyNode(data_to_insert)  # SinglyNode() will validate input
+                setattr(new_node, '_next', current_node.next)  # insert after()
+                setattr(current_node, '_next', new_node)  # insert after()
+                return
+            current_node = current_node.next  # traversal
+        if getattr(self._tail, 'data') == self.target:
+            self.tail = data_to_insert  # # trigger the setter, tail property will validate input
+
+    def insert_before_target(self, target_data, data_to_insert):
+        """The traversal doesn't have to account for head or tail because
+        each is explicitly checked before traversal.
+
+        using current_node.next for traversal accounts for a one node list
+        as .next is not set on the head of a one node list."""
+        self.target = target_data  # trigger setter
+        # insert before first occurence
+        if getattr(self._head, 'data') == self._target:
+            self.head = data_to_insert  # trigger the setter, head property will validate input
+            return
+        current_node = getattr(self, '_head')  # traversal
+        while current_node: # traversal
+            if current_node.next.data == self._target:  # peek
+                #node = current_node.next  # the node containing the target
+                new_node = SinglyNode(data_to_insert)
+                setattr(new_node, '_next', current_node.next)  # insert_before()
+                setattr(current_node, '_next', new_node)  # insert before()
+                return #node
+            current_node = current_node.next  # traversal
+
+    def search(self, target_data):
         """Searches each node's data in a linked list until the first occurrence of
         the target is found.
 
@@ -382,20 +434,13 @@ class SinglyLinkedList:
             >>> llist.search("third node")
             True
         """
-        if target_data:
-            #  TODO: data validation
-            validated_data = target_data
-            if self._head:
-                current_node = self._head
-                while current_node:
-                    if current_node.data == validated_data:
-                        return True
-                    current_node = current_node.next
-                return False
-            raise ValueError("Cannot perform search: The list is empty.")
-        raise ValueError("This data is unable to be target.")
-
-
+        self.target = target_data  # trigger the setter
+        current_node = self._head
+        while current_node:
+            if current_node.data == self._target:
+                return True
+            current_node = current_node.next
+        return False
 
     def remove(self, target_data: Any | SinglyNode) -> None:
         """Removes the first occurrence of a node with the specified target data from the linked list.
@@ -450,29 +495,26 @@ class SinglyLinkedList:
             (SinglyNode(data='2 node'), None, SinglyNode(data='2 node'))
         """
 
-        if not self._head:
-            raise ValueError("The linked list is empty.")
-
         self.target = target_data  # trigger the setter
 
-        if self._head is self._target:
-            # node = self._head
+        if getattr(self._head, 'data') == self._target:
+            # node = self._head  # the node to be removed
             self.remove_head()
             return #node
-        if self._tail is self._target:
-            # node = self._tail
+
+        current_node = getattr(self, '_head')  # traversal
+        while current_node.next is not self._tail:  # traversal (tail already checked)
+            if current_node.next.data == self._target:  # peek
+                #node = current_node.next  # the node to be removed
+                setattr(current_node, '_next', current_node.next.next)  # remove()
+                return #node
+            current_node = current_node.next  # traversal
+
+        if getattr(self._tail, 'data') == self._target:
+            # node = self._tail  # the node to be removed
             self.remove_tail()
             return #node
 
-        current_node = self._head
-        while current_node.next:
-            if current_node.next is self._target:
-                #node = current_node.next
-                setattr(current_node, '_next', current_node.next.next)
-                return # node
-            current_node = current_node.next
-
-
     def remove_head(self) -> None:
         """Removes the head node from the linked list.
 
